require "omniauth-oauth2"

module OmniAuth
  module Strategies
    class Neteasy < OmniAuth::Strategies::OAuth2
      option :client_options, {
        :site           => "https://api.t.163.com",
        :authorize_url  => "/oauth2/authorize",
        :token_url      => "/oauth2/access_token"
      }
      option :token_params, {
        :parse          => :json
      }

      uid do
        raw_info['id']
      end

      info do
        {
          :nickname     => raw_info['screen_name'],
          :name         => raw_info['name'],
          :location     => raw_info['location'],
          :image        => raw_info['profile_image_url'],
          :description  => raw_info['description'],
          :urls => {
            'neteasy_weibo'  => "http://t.163.com/#{raw_info['id']}",
          }
        }
      end

      extra do
        {
          :raw_info => raw_info
        }
      end

      # https://api.t.163.com/users/show.json?oauth_token=6df96bf00f4cdd2a90c1df7077ae698a
      def raw_info
        access_token.options[:mode] = :query
        access_token.options[:param_name] = 'access_token'
        @raw_info ||= access_token.get("/users/show.json").parsed
      end

      ##
      # You can pass +display+, +with_offical_account+ or +state+ params to the auth request, if
      # you need to set them dynamically. You can also set these options
      # in the OmniAuth config :authorize_params option.
      #
      def authorize_params
        super.tap do |params|
          %w[display state].each do |v|
            if request.params[v]
              params[v.to_sym] = request.params[v]

              # to support omniauth-oauth2's auto csrf protection
              session['omniauth.state'] = params[:state] if v == 'state'
            end
          end
        end
      end

    end
  end
end

OmniAuth.config.add_camelization "neteasy", "Neteasy"
